=== modified file 'bzrlib/controldir.py'
--- bzrlib/controldir.py	2011-03-05 02:54:47 +0000
+++ bzrlib/controldir.py	2011-03-10 12:35:32 +0000
@@ -713,7 +713,7 @@
     _formats = []
     """The registered control formats - .bzr, ....
 
-    This is a list of ControlDirFormat objects.
+    This is a list of _ObjectGetters of ControlDirFormat objects.
     """
 
     _server_probers = []
@@ -773,10 +773,18 @@
 
     @classmethod
     def register_format(klass, format):
-        """Register a format that does not use '.bzr' for its control dir.
-
-        """
-        klass._formats.append(format)
+        """Register a control dir format.
+
+        """
+        klass._formats.append(registry._ObjectGetter(format))
+
+    @classmethod
+    def register_lazy_format(klass, module_name, member_name):
+        """Lazily register a control dir format.
+
+        """
+        klass._formats.append(registry._LazyObjectGetter(
+            module_name, member_name))
 
     @classmethod
     def register_prober(klass, prober):
@@ -815,7 +823,7 @@
     def known_formats(klass):
         """Return all the known formats.
         """
-        return set(klass._formats)
+        return set([objgetter.get_obj() for objgetter in klass._formats])
 
     @classmethod
     def find_format(klass, transport, _server_formats=True):

=== modified file 'bzrlib/tests/test_controldir.py'
--- bzrlib/tests/test_controldir.py	2011-02-24 16:23:16 +0000
+++ bzrlib/tests/test_controldir.py	2011-03-10 12:35:32 +0000
@@ -77,3 +77,30 @@
         formats = self.registry._get_all()
         self.assertEquals(1, len(formats))
         self.assertIsInstance(formats[0], SampleExtraComponentFormat)
+
+
+class ControlDirFormatTest1(controldir.ControlDirFormat):
+    """A test control dir format."""
+
+
+class TestControlDirFormat(tests.TestCaseWithTransport):
+    """Tests for the ControlDirFormat facility."""
+
+    def test_register_unregister_format(self):
+        format = ControlDirFormatTest1()
+        controldir.ControlDirFormat.register_format(format)
+        self.assertTrue(format in controldir.ControlDirFormat.known_formats())
+        controldir.BzrDirFormat.unregister_format(format)
+        self.assertFalse(format in controldir.ControlDirFormat.known_formats())
+
+    def test_register_unregister_format_lazy(self):
+        controldir.ControlDirFormat.register_lazy_format(
+            "bzrlib.tests.test_controldir", "ControlDirFormatTest1")
+        self.assertTrue(bool([
+            format.__class__ is ControlDirFormatTest1 for format in
+            controldir.ControlDirFormat.known_formats()]))
+        controldir.BzrDirFormat.unregister_lazy_format(
+            "bzrlib.tests.test_controldir", "ControlDirFormatTest1")
+        self.assertFalse(bool([
+            format.__class__ is ControlDirFormatTest1 for format in
+            controldir.ControlDirFormat.known_formats()]))

